<?php

/**
 * @file
 * Reporting functionality for helpfulness module.
 */

/**
 * Implements helpfulness_report_form().
 */
function helpfulness_report_form($form, &$form_state) {

  if (isset($form_state['storage']) && $form_state['storage']['ask_confirm']) {
    return confirm_form(
      $form,
      t('Are you sure you want to delete the selected items?'),
      'admin/reports/helpfulness',
      helpfulness_get_confirm_form_description($form_state['storage']['original_form'])
    );
  }

  return helpfulness_get_report_form();

}

/**
 * Implements helpfulness_get_report_form().
 */
function helpfulness_get_report_form() {

  // Build the array for the header of the table.
  $header = array();
  if (variable_get('helpfulness_report_option_display_username', 0)) {
    $header['uid'] = array(
      'data'  => t('User ID'),
      'field' => 'uid',
    );
  }
  if (variable_get('helpfulness_report_option_display_helpfulness', 1)) {
    $header['helpfulness'] = array(
      'data'  => t('Helpful'),
      'field' => 'helpfulness',
    );
  }
  if (variable_get('helpfulness_report_option_display_message', 1)) {
    $header['message'] = array(
      'data'  => t('Message'),
      'field' => 'message',
    );
  }
  if (variable_get('helpfulness_report_option_display_base_url', 0)) {
    $header['base_url'] = array(
      'data'  => t('Base URL'),
      'field' => 'base_url',
    );
  }
  if (variable_get('helpfulness_report_option_display_system_path', 1)) {
    $header['system_path'] = array(
      'data'  => t('System Path'),
      'field' => 'system_path',
    );
  }
  if (variable_get('helpfulness_report_option_display_alias', 0)) {
    $header['path_alias'] = array(
      'data'  => t('Alias'),
      'field' => 'path_alias',
    );
  }
  if (variable_get('helpfulness_report_option_display_date', 1)) {
    $header['date'] = array(
      'data'  => t('Date'),
      'field' => 'timestamp',
    );
  }
  if (variable_get('helpfulness_report_option_display_time', 0)) {
    $header['time'] = t('Time');
  }
  if (variable_get('helpfulness_report_option_display_useragent', 0)) {
    $header['useragent'] = array(
      'data'  => t('Browser'),
      'field' => 'useragent',
    );
  }

  // Setting the sort conditions.
  if (isset($_GET['sort']) && isset($_GET['order'])) {
    // Sort it Ascending or Descending?
    if ($_GET['sort'] == 'asc') {
      $sort = 'ASC';
    }
    else {
      $sort = 'DESC';
    }

    // Which column will be sorted.
    switch ($_GET['order']) {
      case 'User ID':
        $order = 'uid';
        break;

      case 'Helpful':
        $order = 'helpfulness';
        break;

      case 'Message':
        $order = 'message';
        break;

      case 'Base URL':
        $order = 'base_url';
        break;

      case 'System Path':
        $order = 'system_path';
        break;

      case 'Alias':
        $order = 'path_alias';
        break;

      case 'Date':
        $order = 'timestamp';
        break;

      case 'Browser':
        $order = 'useragent';
        break;

      default:
        $order = 'timestamp';
    }
  }
  else {
    $sort = 'ASC';
    $order = 'timestamp';
  }

  // For all feedbacks retrieved from the db.
  $feedbacks = array();

  // Build the query.
  $query = db_select('helpfulness', 'hf')
           ->fields('hf')
           ->orderBy($order, $sort)
           ->extend('TableSort');

  // Fetch all results from the db.
  foreach ($query->execute() as $row) {
    $feedbacks[] = array(
      'fid'         => $row->fid,
      'status'      => $row->status,
      'uid'         => $row->uid,
      'helpfulness' => $row->helpfulness,
      'message'     => $row->message,
      'useragent'   => $row->useragent,
      'timestamp'   => $row->timestamp,
      'system_path' => $row->system_path,
      'path_alias'  => $row->path_alias,
      'base_url'    => $row->base_url,
    );
  }

  // Build the rows for the table.
  date_default_timezone_set(drupal_get_user_timezone());
  $options_open = array();
  $options_archived = array();

  foreach ($feedbacks as $feedback) {
    $option = array();

    if ($feedback['uid'] == 0) {
      $username = t('Anonymous');
    }
    else {
      $tmp_user = user_load($feedback['uid']);
      $username = $tmp_user->name;
    }
    $option['uid'] = $feedback['uid'] . ' (' . $username . ')';
    $option['helpfulness'] = ($feedback['helpfulness']) ? 'Yes' : 'No';
    $option['message'] = $feedback['message'];
    $option['system_path'] = $feedback['system_path'];
    $option['path_alias'] = $feedback['path_alias'];
    $option['base_url'] = $feedback['base_url'];
    $option['date'] = format_date($feedback['timestamp'], 'custom', 'Y-m-d');
    $option['time'] = format_date($feedback['timestamp'], 'custom', 'H:i');
    $option['useragent'] = array(
      'data' => array(
        '#markup' => $feedback['useragent'],
        '#prefix' => '<div class="useragent_description">',
        '#suffix' => '</div>',
      ),
    );

    // Add this feedback to the appropriate table.
    switch ($feedback['status']) {
      case HELPFULNESS_STATUS_OPEN:
        $options_open[$feedback['fid']] = $option;
        break;

      case HELPFULNESS_STATUS_ARCHIVED:
        $options_archived[$feedback['fid']] = $option;
        break;

    }

  }

  // Update actions.
  $form['update_action'] = array(
    '#type'     => 'select',
    '#title'    => t('Update options:'),
    '#options'  => array(
      ''                          => t('Please Select...'),
      HELPFULNESS_STATUS_OPEN     => t('Set as "new"'),
      HELPFULNESS_STATUS_ARCHIVED => t('Archive'),
      HELPFULNESS_STATUS_DELETED  => t('Delete'),
    ),
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type'  => 'submit',
    '#value' => t('Update'),
  );

  // Include the css for formatting of the user agent string.
  $form['#attached']['css'] = array(
    drupal_get_path('module', 'helpfulness') . '/css/helpfulness.css',
  );

  // Add the tables for open feedbacks to the form.
  $form['helpfulness_feedback_open'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('New feedback'),
    '#collapsible' => TRUE,
    '#collapsed'   => FALSE,
  );

  $form['helpfulness_feedback_open']['helpfulness_feedbacks_open_table'] = array(
    '#type'    => 'tableselect',
    '#header'  => $header,
    '#options' => $options_open,
    '#empty'   => t('No new feedback found'),
  );

  // Add the tables for archived feedbacks to the form.
  $form['helpfulness_feedbacks_archived'] = array(
    '#type'        => 'fieldset',
    '#title'       => t('Archived feedback'),
    '#collapsible' => TRUE,
    '#collapsed'   => FALSE,
  );

  $form['helpfulness_feedbacks_archived']['helpfulness_feedbacks_archived_table'] = array(
    '#type'    => 'tableselect',
    '#header'  => $header,
    '#options' => $options_archived,
    '#empty'   => t('No archived feedback found'),
  );

  return $form;
}

/**
 * Implements helpfulness_get_confirm_form_description().
 */
function helpfulness_get_confirm_form_description($original_form) {

  $open_fids = array_diff($original_form['helpfulness_feedbacks_open_table'], array("0"));
  $archived_fids = array_diff($original_form['helpfulness_feedbacks_archived_table'], array("0"));
  $count = count($open_fids) + count($archived_fids);

  $plural = '';
  if ($count > 1) {
    $plural = 's';
  }

  $description = '<p>' . t('You are about to DELETE the @c feedback item@p you had selected.', array('@c' => $count, '@p' => $plural)) . '</p>';

  $description .= '<p><strong>' . t('This can not be undone.') . '</strong></p>';

  return $description;
}

/**
 * Implements helpfulness_report_form_submit().
 */
function helpfulness_report_form_submit($form, &$form_state) {
  if (isset($form_state['storage'])) {
    $values = $form_state['storage']['original_form'];
  }
  else {
    $values = $form_state['values'];
  }

  $open_fids = array_diff($values['helpfulness_feedbacks_open_table'], array("0"));
  $archived_fids = array_diff($values['helpfulness_feedbacks_archived_table'], array("0"));

  if (empty($open_fids) && empty($archived_fids)) {
    drupal_set_message(t('Please select the items you would like to update.'), 'error');
    return;
  }

  $action = $values['update_action'];

  if (!isset($form_state['storage']) && $action == HELPFULNESS_STATUS_DELETED) {
    $form_state['storage']['ask_confirm'] = TRUE;
    $form_state['storage']['original_form'] = $values;
    $form_state['rebuild'] = TRUE;
  }
  else {
    helpfulness_process_update_action($action, $open_fids);
    helpfulness_process_update_action($action, $archived_fids);
    drupal_set_message(t('Your selected items have been updated.'));
  }

}

/**
 * Implements helpfulness_process_update_action().
 */
function helpfulness_process_update_action($action, $selected_fids) {
  if (empty($selected_fids)) {
    return;
  }
  db_update('helpfulness')
    ->fields(array(
      'status' => $action,
    ))
    ->condition('fid', $selected_fids, 'IN')
    ->execute();
}
